@@ -19,15 +19,20 @@ from .interface import (
     configure,
     configure_environ
 )
-from .exception import InterfaceError, Error, UniqueError
+from .exception import (
+    Error,
+    InterfaceError,
+    UniqueError,
+    TableError,
+    FieldError,
+    UniqueError,
+    CloseError,
+)
+
 from . import utils
 
 
-__version__ = '4.1.1'
-
-
-# get rid of "No handler found" warnings (cribbed from requests)
-#logging.getLogger(__name__).addHandler(logging.NullHandler())
+__version__ = '4.2.0'
 
 
 def transaction(connection_name="", **kwargs):
