@@ -20,7 +20,7 @@ import psycopg2.extras
 import psycopg2.extensions
 
 # first party
-from ..base import SQLInterface, SQLConnection
+from ..sql import SQLInterface, SQLConnection
 from ...compat import *
 from ...utils import get_objects
 from ...exception import (
@@ -88,14 +88,14 @@ class StringType(object):
         return val
 
 
-class Connection(SQLConnection, psycopg2.extensions.connection):
+class PostgreSQLConnection(SQLConnection, psycopg2.extensions.connection):
 #class Connection(SQLConnection, psycopg2.extras.LoggingConnection):
     """
     http://initd.org/psycopg/docs/advanced.html
     http://initd.org/psycopg/docs/extensions.html#psycopg2.extensions.connection
     """
     def __init__(self, *args, **kwargs):
-        super(Connection, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
         # http://initd.org/psycopg/docs/connection.html#connection.autocommit
         self.autocommit = True
@@ -108,7 +108,6 @@ class Connection(SQLConnection, psycopg2.extensions.connection):
         # https://www.psycopg.org/docs/extensions.html#psycopg2.extensions.register_adapter
         psycopg2.extensions.register_adapter(dict, DictType.adapt)
 
-
         # http://initd.org/psycopg/docs/connection.html#connection.set_client_encoding
         # https://www.postgresql.org/docs/current/static/multibyte.html
         # > The default is the encoding defined by the database
@@ -123,9 +122,7 @@ class PostgreSQL(SQLInterface):
 
     val_placeholder = '%s'
 
-    connection_pool = None
-
-    _connection = None
+    _connection_pool = None
 
     def _connect(self, connection_config):
         database = connection_config.database
@@ -141,14 +138,13 @@ class PostgreSQL(SQLInterface):
             'pool_class',
             'psycopg2.pool.SimpleConnectionPool'
         )
-        async_conn = int(connection_config.options.get('async', 0))
 
         _, pool_class = get_objects(pool_class_name)
 
-        self.log("connecting using pool class {}".format(pool_class_name))
+        self.log_debug("connecting using pool class {}".format(pool_class_name))
 
         # http://initd.org/psycopg/docs/module.html#psycopg2.connect
-        self.connection_pool = pool_class(
+        self._connection_pool = pool_class(
             minconn,
             maxconn,
             dbname=database,
@@ -158,49 +154,40 @@ class PostgreSQL(SQLInterface):
             port=port,
             cursor_factory=psycopg2.extras.RealDictCursor,
             #cursor_factory=LoggingCursor,
-            connection_factory=Connection,
+            connection_factory=PostgreSQLConnection,
         )
 
-        # hack for sync backwards compatibility with transactions
-        if not async_conn:
-            self._connection = self.connection_pool.getconn()
+    def _get_connection(self):
+        connection = self._connection_pool.getconn()
 
-    def free_connection(self, connection):
-        if not self.connected: return
-
-        if self._connection:
-            self.log("freeing sync connection")
-
-        else:
-            self.log("freeing async connection {}", id(connection))
-            self.connection_pool.putconn(connection)
-
-    def get_connection(self):
-        if not self.connected: self.connect()
-
-        connection = None
-        if self._connection:
-            self.log("getting sync connection")
-            connection = self._connection
-
-        else:
-            connection = self.connection_pool.getconn()
-            self.log("getting async connection {}", id(connection))
+        connection_id = id(connection)
 
         # change the connection readonly status if they don't match
-        if connection.readonly != self.connection_config.readonly:
+        if bool(connection.readonly) != bool(self.connection_config.readonly):
             # https://www.psycopg.org/docs/connection.html#connection.readonly
+            self.log_warning([
+                f"Changing connection {connection_id:02x}",
+                f"to readonly={self.connection_config.readonly}",
+                f"from readonly={connection.readonly}",
+            ])
             connection.readonly = self.connection_config.readonly
+
         return connection
 
-    def _close(self):
-        self.connection_pool.closeall()
-        if self._connection:
-            self._connection.close()
-            self._connection = None
-            self.connection_pool = None
+    def _free_connection(self, connection):
+        # if an error was handled there is a chance that the connection was reset
+        # and we don't want to put a dead connection back into the pool
+        if connection.closed:
+            self.log_warning(f"Discarding pool connection {id(connection):02x} because it is closed")
 
-    def _readonly(self, readonly):
+        else:
+            self._connection_pool.putconn(connection)
+
+    def _close(self):
+        self._connection_pool.closeall()
+        self._connection_pool = None
+
+    def _readonly(self, readonly, **kwargs):
         """readonly setting is handled when you grab the connection from get_connection()
         so this method does nothing"""
         pass
@@ -213,13 +200,16 @@ class PostgreSQL(SQLInterface):
             query_str += ' AND tablename = %s'
             query_args.append(str(table_name))
 
-        ret = self.query(query_str, *query_args, **kwargs)
+        ret = self._query(query_str, *query_args, **kwargs)
         # http://www.postgresql.org/message-id/CA+mi_8Y6UXtAmYKKBZAHBoY7F6giuT5WfE0wi3hR44XXYDsXzg@mail.gmail.com
         return [r['tablename'] for r in ret]
 
     def _delete_table(self, schema, **kwargs):
+        """
+        https://www.postgresql.org/docs/current/sql-droptable.html
+        """
         query_str = 'DROP TABLE IF EXISTS {} CASCADE'.format(self._normalize_table_name(schema))
-        ret = self.query(query_str, ignore_result=True, **kwargs)
+        ret = self._query(query_str, ignore_result=True, **kwargs)
 
     def _get_fields(self, table_name, **kwargs):
         """return all the fields for the given schema"""
@@ -263,8 +253,8 @@ class PostgreSQL(SQLInterface):
             '  AND a.attnum > 0',
             'ORDER BY a.attnum ASC',
         ]
-        query_str = os.linesep.join(query_str)
-        fields = self.query(query_str, *query_args, **kwargs)
+        query_str = "\n".join(query_str)
+        fields = self._query(query_str, *query_args, **kwargs)
 
         pg_types = {
             "float4": float,
@@ -319,9 +309,9 @@ class PostgreSQL(SQLInterface):
             'ORDER BY',
             '  tbl.relname, i.relname',
         ]
-        query_str = os.linesep.join(query_str)
+        query_str = "\n".join(query_str)
 
-        indexes = self.query(query_str, 'r', str(schema), **kwargs)
+        indexes = self._query(query_str, 'r', str(schema), **kwargs)
 
         # massage the data into more readable {index_name: fields} format
         for idict in indexes:
@@ -333,22 +323,6 @@ class PostgreSQL(SQLInterface):
 
         return ret
 
-    def _set_index(self, schema, name, field_names, **index_options):
-        """
-        NOTE -- we set the index name using <table_name>_<name> format since indexes have to have
-        a globally unique name in postgres
-
-        http://www.postgresql.org/docs/9.1/static/sql-createindex.html
-        """
-        query_str = 'CREATE {}INDEX {} ON {} USING BTREE ({})'.format(
-            'UNIQUE ' if index_options.get('unique', False) else '',
-            self._normalize_name("{}_{}".format(schema, name)),
-            self._normalize_table_name(schema),
-            ', '.join(map(self._normalize_name, field_names))
-        )
-
-        return self.query(query_str, ignore_result=True, **index_options)
-
     def _normalize_field_SQL(self, schema, field_name, symbol):
         format_field_name = self._normalize_name(field_name)
         format_val_str = self.val_placeholder
@@ -515,9 +489,17 @@ class PostgreSQL(SQLInterface):
                     #'relation "<TABLE-NAME>" does not exit'
                     e = TableError(e)
 
+        elif isinstance(e, psycopg2.errors.AdminShutdown):
+            e = CloseError(e)
+
+        elif isinstance(e, psycopg2.errors.InFailedSqlTransaction):
+            e = CloseError(e)
+
         elif isinstance(e, psycopg2.IntegrityError):
             e = UniqueError(e)
+
         else:
             e = super().create_error(e, **kwargs)
+
         return e
 
