@@ -44,16 +44,7 @@ from ..exception import (
 )
 
 from ..compat import *
-from .base import SQLInterface, SQLConnection
-
-
-class SQLiteRowDict(sqlite3.Row):
-    def get(self, k, default_val=None):
-        try:
-            r = self[k]
-        except KeyError:
-            r = default_val
-        return r
+from .sql import SQLInterface, SQLConnection
 
 
 class SQLiteConnection(SQLConnection, sqlite3.Connection):
@@ -209,14 +200,20 @@ class SQLite(SQLInterface):
         return connection_config
 
     def _connect(self, connection_config):
+        """
+        https://docs.python.org/3.11/library/sqlite3.html#sqlite3.connect
+        """
         path = connection_config.path
 
         # https://docs.python.org/2/library/sqlite3.html#default-adapters-and-converters
         options = {
             'isolation_level': None,
+            #'isolation_level': "IMMEDIATE",
+            #'isolation_level': "EXCLUSIVE",
             'detect_types': sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES,
             'factory': SQLiteConnection,
             'check_same_thread': True, # https://stackoverflow.com/a/2578401/5006
+            #'timeout': 100,
         }
         option_keys = list(options.keys()) + ['timeout', 'cached_statements']
         for k in option_keys:
@@ -225,6 +222,7 @@ class SQLite(SQLInterface):
 
         try:
             self._connection = sqlite3.connect(path, **options)
+            self.log_debug(f"Connected to connection 0x{id(self._connection):02x}")
 
         except sqlite3.DatabaseError as e:
             path_d = os.path.dirname(path)
@@ -237,7 +235,7 @@ class SQLite(SQLInterface):
                 self._connection = sqlite3.connect(path, **options)
 
         # https://docs.python.org/2/library/sqlite3.html#row-objects
-        self._connection.row_factory = SQLiteRowDict
+        self._connection.row_factory = sqlite3.Row
         # https://docs.python.org/2/library/sqlite3.html#sqlite3.Connection.text_factory
         self._connection.text_factory = StringType.adapt
 
@@ -245,9 +243,6 @@ class SQLite(SQLInterface):
         # to be ran through the converter, not sure why
         sqlite3.register_converter(StringType.FIELD_TYPE, StringType.adapt)
 
-        #sqlite3.register_adapter(decimal.Decimal, NumericType.adapt)
-        #sqlite3.register_converter('NUMERIC', NumericType.convert)
-
         sqlite3.register_adapter(bool, BooleanType.adapt)
         sqlite3.register_converter(BooleanType.FIELD_TYPE, BooleanType.convert)
 
@@ -266,24 +261,29 @@ class SQLite(SQLInterface):
         sqlite3.register_adapter(dict, DictType.adapt)
         sqlite3.register_converter(DictType.FIELD_TYPE, DictType.convert)
 
+    def _configure_connection(self, **kwargs):
         # turn on foreign keys
         # http://www.sqlite.org/foreignkeys.html
-        self._query('PRAGMA foreign_keys = ON', ignore_result=True);
-        self.readonly(self.connection_config.readonly)
+        self._query('PRAGMA foreign_keys = ON', ignore_result=True, **kwargs)
 
-    def get_connection(self):
-        if not self.connected: self.connect()
+        # by default we can read/write, so only bother to run this query if we need to
+        # make the connection actually readonly
+        if self.connection_config.readonly:
+            self._readonly(self.connection_config.readonly, **kwargs)
+
+    def _get_connection(self):
         return self._connection
 
     def _close(self):
         self._connection.close()
         self._connection = None
 
-    def _readonly(self, readonly):
+    def _readonly(self, readonly, **kwargs):
         self._query(
             # https://stackoverflow.com/a/49630725/5006
             'PRAGMA query_only = {}'.format("ON" if readonly else "OFF"),
-            ignore_result=True
+            ignore_result=True,
+            **kwargs
         )
 
     def _get_tables(self, table_name, **kwargs):
@@ -294,23 +294,9 @@ class SQLite(SQLInterface):
             query_str += ' AND name = ?'
             query_args.append(str(table_name))
 
-        ret = self._query(query_str, query_args, **kwargs)
+        ret = self._query(query_str, *query_args, **kwargs)
         return [r['tbl_name'] for r in ret]
 
-    def _set_index(self, schema, name, field_names, **index_options):
-        """
-        https://www.sqlite.org/lang_createindex.html
-        """
-        query_str = "CREATE {}INDEX IF NOT EXISTS '{}_{}' ON {} ({})".format(
-            'UNIQUE ' if index_options.get('unique', False) else '',
-            schema,
-            name,
-            self._normalize_table_name(schema),
-            ', '.join(map(self._normalize_name, field_names))
-        )
-
-        return self._query(query_str, ignore_result=True, **index_options)
-
     def _get_indexes(self, schema, **kwargs):
         """return all the indexes for the given schema"""
         # http://www.sqlite.org/pragma.html#schema
@@ -330,14 +316,17 @@ class SQLite(SQLInterface):
 
     def _delete_tables(self, **kwargs):
         self._query('PRAGMA foreign_keys = OFF', ignore_result=True, **kwargs);
-        ret = super(SQLite, self)._delete_tables(**kwargs)
+        ret = super()._delete_tables(**kwargs)
         self._query('PRAGMA foreign_keys = ON', ignore_result=True, **kwargs);
         return ret
 
     def _delete_table(self, schema, **kwargs):
+        """
+        https://www.sqlite.org/lang_droptable.html
+        """
         #query_str = 'DROP TABLE IF EXISTS {}'.format(str(schema))
         query_str = "DROP TABLE IF EXISTS {}".format(self._normalize_table_name(schema))
-        ret = self._query(query_str, ignore_result=True, **kwargs)
+        ret = self.query(query_str, ignore_result=True, **kwargs)
 
     def create_error(self, e, **kwargs):
         if isinstance(e, sqlite3.OperationalError):
@@ -353,12 +342,20 @@ class SQLite(SQLInterface):
             elif "UNIQUE" in e_msg:
                 e = UniqueError(e)
 
+            elif "database is locked":
+                e = CloseError(e)
+
             else:
                 e = super().create_error(e, **kwargs)
 
         elif isinstance(e, sqlite3.IntegrityError):
             e = UniqueError(e)
 
+        elif isinstance(e, sqlite3.ProgrammingError):
+            e_msg = str(e)
+            if "closed database" in e_msg:
+                e = CloseError(e)
+
         else:
             e = super().create_error(e, **kwargs)
 
