@@ -13,8 +13,10 @@ import math
 import inspect
 import time
 import re
+import weakref
 
 from datatypes.collections import ListIterator
+from datatypes import property as cachedproperty
 
 from . import decorators
 from .utils import make_list, get_objects, make_dict, make_hash
@@ -56,6 +58,14 @@ class Iterator(ListIterator):
 
         self.reset()
 
+        # cleanup
+        weakref.finalize(self, self.__del__)
+
+    def __del__(self):
+        """Whenever this gets garbage collected close everything. This is also the
+        method for weakref.finalize"""
+        self.close()
+
     def has_more(self):
         """Return true if there are more results for this query if the query didn't
         have a LIMIT clause
@@ -88,8 +98,17 @@ class Iterator(ListIterator):
 
         return cursor
 
+    def close(self):
+        if "_cursor" in self.__dict__:
+            try:
+                self._cursor.close()
+
+            except Exception:
+                pass
+
     def reset(self):
         """put all the pieces together to build a generator of the results"""
+        self.close()
         self._cursor = None
         self._cursor_i = 0
 
@@ -232,11 +251,10 @@ class Iterator(ListIterator):
         if self.field_names:
             field_vals = []
             for field_name in self.field_names:
-                fv = d.get(field_name, None)
+                fv = d[field_name]
                 if orm_class:
                     fv = orm_class.schema.fields[field_name].iget(None, fv)
                 field_vals.append(fv)
-            #field_vals = [d.get(fn, None) for fn in self.field_names]
             r = field_vals if len(self.field_names) > 1 else field_vals[0]
 
         else:
@@ -504,25 +522,10 @@ class Query(object):
     bounds_class = Bounds
     iterator_class = Iterator
 
+
     @property
     def interface(self):
-        if not self.orm_class: return None
-        interface = getattr(self, "_interface", None)
-        if not interface:
-            interface = self.orm_class.interface
-            self._interface = interface
-        return interface
-
-    @interface.setter
-    def interface(self, interface):
-        self._interface = interface
-
-    @interface.deleter
-    def interface(self):
-        try:
-            del self._interface
-        except AttributeError:
-            pass
+        return self.orm_class.interface if self.orm_class else None
 
     @property
     def schema(self):
@@ -560,7 +563,7 @@ class Query(object):
         self.kwargs = kwargs
 
     def reset(self):
-        self.interface = None
+        #self.interface = None
         self._ifilter = None
         self.fields_set = self.fields_set_class()
         self.fields_select = self.fields_select_class()
@@ -1003,12 +1006,16 @@ class Query(object):
 
     def __deepcopy__(self, memodict={}):
         instance = type(self)(self.orm_class)
-        ignore_keys = set(["_interface"])
+        ignore_keys = set(["_interface", "interface"])
         for key, val in self.__dict__.items():
             if key not in ignore_keys:
                 setattr(instance, key, copy.deepcopy(val, memodict))
         return instance
 
     def __str__(self):
-        return self.render()
+        try:
+            return self.render()
+
+        except AttributeError:
+            return super().__str__()
 
